/******************************************************************************
 * $Id: ogrdwgdriver.cpp 10646 2007-01-18 02:38:10Z warmerdam $
 *
 * Project:  OpenGIS Simple Features Reference Implementation
 * Purpose:  Implements OGRDWGDriver class.
 * Author:   Frank Warmerdam, warmerdam@pobox.com
 *
 ******************************************************************************
 * Copyright (c) 2005, Frank Warmerdam <warmerdam@pobox.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 ****************************************************************************/

#include "ogr_dwg.h"
#include "cpl_conv.h"

CPL_CVSID("$Id: ogrdwgdriver.cpp 10646 2007-01-18 02:38:10Z warmerdam $");

//gok:
/************************************************************************/
/* Define a module map for statically linked modules:                   */
/************************************************************************/
#ifndef _TOOLKIT_IN_DLL_
	ODRX_DECLARE_STATIC_MODULE_ENTRY_POINT(BitmapModule);
#pragma comment(lib, "Gs_WinBitmap.lib")
	// to handle bmp preview
	ODRX_DECLARE_STATIC_MODULE_ENTRY_POINT(OdRasterProcessingServicesImpl); 
	ODRX_DECLARE_STATIC_MODULE_ENTRY_POINT(OdBagFilerModuleImpl);
	// DWF export
	ODRX_DECLARE_STATIC_MODULE_ENTRY_POINT(ModelerModule);
	ODRX_DECLARE_STATIC_MODULE_ENTRY_POINT(OdRecomputeDimBlockModule);
	
ODRX_BEGIN_STATIC_MODULE_MAP()
   ODRX_DEFINE_STATIC_APPMODULE  (DD_T("WinBitmap.gs"),     BitmapModule)
   ODRX_DEFINE_STATIC_APPLICATION(DD_T("RasterProcessor"),	OdRasterProcessingServicesImpl) // to handle bmp preview
   ODRX_DEFINE_STATIC_APPLICATION(DD_T("ModelerGeometry"),   ModelerModule)
   ODRX_DEFINE_STATIC_APPLICATION(DD_T("RecomputeDimBlock"), OdRecomputeDimBlockModule)
   ODRX_DEFINE_STATIC_APPLICATION(DD_T("OdBagFiler.drx"),	OdBagFilerModuleImpl)
ODRX_END_STATIC_MODULE_MAP() 
#endif

OdStaticRxObject<MyServices> svcs;

/************************************************************************/
/*                            OGRDWGDriver()                            */
/************************************************************************/

OGRDWGDriver::OGRDWGDriver( const char *pszName )

{
    osOutClass = pszName;
	//gok:
#ifndef _TOOLKIT_IN_DLL_
	ODRX_INIT_STATIC_MODULE_MAP();
#endif
	odInitialize(&svcs);
}

/************************************************************************/
/*                            ~OGRDWGDriver()                            */
/************************************************************************/

OGRDWGDriver::~OGRDWGDriver()

{
	//gok:
	::odUninitialize();
}

/************************************************************************/
/*                              GetName()                               */
/************************************************************************/

const char *OGRDWGDriver::GetName()

{
    return osOutClass;
}

/************************************************************************/
/*                                Open()                                */
/************************************************************************/

OGRDataSource *OGRDWGDriver::Open( const char * pszFilename,
                                     int bUpdate )

{
	OGRWritableDWGDataSource *pDS = new OGRWritableDWGDataSource(pszFilename);
	char* pOptions = "rb";
	if ( !pDS->Create(pszFilename, &pOptions) )
	{
		delete pDS;
		pDS = NULL;		
	}
	return pDS;
#if 0
    if( EQUAL(CPLGetExtension(pszFilename),"dxf") 
        || EQUAL(CPLGetExtension(pszFilename),"dwg") )
        CPLError( CE_Failure, CPLE_NotSupported, 
                  "DXF/DWG reading not yet implemented." );
    return NULL;
#endif
}

/************************************************************************/
/*                          CreateDataSource()                          */
/************************************************************************/

OGRDataSource *OGRDWGDriver::CreateDataSource( const char * pszName,
                                              char **papszOptions )

{
    OGRWritableDWGDataSource     *poDS;

    poDS = new OGRWritableDWGDataSource( osOutClass );

    if( !poDS->Create( pszName, papszOptions ) )
    {
        delete poDS;
        poDS = NULL;
    }

    return poDS;
}

/************************************************************************/
/*                           TestCapability()                           */
/************************************************************************/

int OGRDWGDriver::TestCapability( const char * pszCap )

{
    if( EQUAL(pszCap,ODrCCreateDataSource) )
        return TRUE;
    else
        return FALSE;
}

/************************************************************************/
/*                           RegisterOGRDXFDWG()                        */
/************************************************************************/

void RegisterOGRDXFDWG()

{
    OGRSFDriverRegistrar::GetRegistrar()->RegisterDriver( 
        new OGRDWGDriver( "DWG" ) );
    OGRSFDriverRegistrar::GetRegistrar()->RegisterDriver( 
        new OGRDWGDriver( "DXF" ) );
}

